#!/bin/sh
#
# Copyright 2020. Huawei Technologies Co., Ltd. All rights reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
# V1.0.0
#
######################################################################################################
# 使用方法:
# 1. 复制此脚本到您Xcode工程的Target->Build Phases->Run Script中
# 2. 在AppGallery Connect网站获取您项目的信息并填入下面对应位置
# Usage:
# 1. Copy the script to the Target->Build Phases->Run Scrip in your Xcode project.
# 2. Get the information of your project on the AppGallery Connect website and fill in the parameters below.
######################################################################################################
#
# 在此填入您应用的APP ID
# Fill in the APP ID of your application here.
APP_ID="<your_app_id>"
#
# 请填入您应用的Client ID
# Fill in the Client ID of your application here.
CLIENT_ID="<your_client_id>"
#
# 请填入您应用的Client密钥
# Fill in the Client Secret of your application here.
CLIENT_SECRET="<your_client_secret>"
#
# 请填入您应用的AGC地址
# Fill in the agcgw url of your application here.
AGCGW_URL="<your_agcgw_url>"
#
# 请填入您应用的HA地址
# Fill in the collector url of your application here.
COLLECTOR_URL="<your_collector_url>"
#
######################################################################################################
# 通过Xcode上传符号表的配置 
# Configuration for uploading dSYM in Xcode
######################################################################################################
# 
# 模拟器编译是否上传，1=上传 0=不上传
# Whether to upload when building in simulator, 1=upload 0=no upload.
UPLOAD_SIMULATOR_SYMBOLS=0
#
# Debug模式编译是否上传，1＝上传 0＝不上传
# Whether to upload when building in debug mode, 1=upload 0=no upload.
UPLOAD_DEBUG_SYMBOLS=0
#
# 仅Archive操作时上传, 1=仅Archive上传 0=Archive和Release模式编译都上传
# Whether to upload only when building in archive mode, 1=archive only, 0=release and archive.
UPLOAD_ARCHIVE_ONLY=0
#
######################################################################################################


function exitWithMessage(){
    echo "--------------------------------"
    echo "❌ ${1}"
    echo "--------------------------------"
    exit 0
}

function get_json_value(){
    if [[ -z "$3" ]]; then
        echo "$1" | python -c "import sys, json; reload(sys); sys.setdefaultencoding('utf8'); print json.load(sys.stdin)['$2']"
    else
        echo "$1" | python -c "import sys, json; reload(sys); sys.setdefaultencoding('utf8'); print json.load(sys.stdin)['$2']['$3']"
    fi
}

function get_json_headers(){
    echo "$1" | python -c "import sys, json, urllib; reload(sys); sys.setdefaultencoding('utf8'); dic_json=json.load(sys.stdin)['headers']; print [urllib.quote(str(k)+':'+str(v)) for k,v in dic_json.items()];"
}

function sha256sum() { openssl sha256 "$@" | awk '{print $2}'; }

function get_file_size() { ls -l "$@" | awk '{print $5}'; }

function urlencode() { 
    python -c 'import urllib, sys; print urllib.quote(sys.argv[1], sys.argv[2])' "$1" "$urlencode_safe"
}

function urldecode() { 
    python -c 'import urllib, sys; print urllib.unquote(sys.argv[1])' "$1"
}

function upload_dsym(){
    DSYM_FOLDER="$1"

    echo "Checking parameters ..."
    if [[ -z "$APP_ID" ]]; then
        exitWithMessage "Error: APP_ID not defined."
    fi
    if [[ -z "$CLIENT_ID" ]]; then
        exitWithMessage "Error: CLIENT_ID not defined."
    fi
    if [[ -z "$CLIENT_SECRET" ]]; then
        exitWithMessage "Error: CLIENT_SECRET not defined."
    fi
    if [[ -z "$AGCGW_URL" ]]; then
        exitWithMessage "Error: AGCGW_URL not defined."
    fi
    if [[ -z "$COLLECTOR_URL" ]]; then
        exitWithMessage "Error: COLLECTOR_URL not defined."
    fi
    echo "✅ Check parameters successfully"



    echo "Getting token..."
    AT_URL="https://${AGCGW_URL}/agc/apigw/oauth2/v1/token"
    AT_PARAM='{"grant_type":"client_credentials","client_id":"'"${CLIENT_ID}"'","client_secret":"'"${CLIENT_SECRET}"'"}'
    AT_RESULT=$(curl -H "Content-type: application/json" -X POST -d $AT_PARAM $AT_URL)
    AT=$(get_json_value "${AT_RESULT}" "access_token")
    if [[ -z "$AT" ]]; then
        exitWithMessage "Error: get token failed.\n${AT_RESULT}"
    else
        echo "✅ Get token successfully."
    fi
    TOKEN="Bearer $AT"

    echo "Scaning dSYM folder: ${DSYM_FOLDER} ..."

    for dsymFile in $(find "$DSYM_FOLDER" -name '*.dSYM'); do
        DSYM_FILE_NAME=${dsymFile##*/}
        DSYM_SYMBOL_ZIP_FILE_NAME="${DSYM_FILE_NAME}.zip"
        DSYM_SYMBOL_ZIP_FILE_NAME="${DSYM_SYMBOL_ZIP_FILE_NAME// /_}"
        DSYM_SYMBOL_ZIP_FILE=${DSYM_FOLDER}/${DSYM_SYMBOL_ZIP_FILE_NAME}
    done

    if [[ -z "$dsymFile" ]]; then
        exitWithMessage "Error: No .dSYM found in DSYM FOLDER."
    else
        echo "✅ Found dSYM file: $dsymFile"
    fi

    echo "Zipping dSYM file..."
    zip -r -j $DSYM_SYMBOL_ZIP_FILE $dsymFile -x *.plist -x *.zip
    SHA=$(sha256sum $DSYM_SYMBOL_ZIP_FILE)
    SIZE=$(get_file_size $DSYM_SYMBOL_ZIP_FILE)
    echo $SIZE
    if [[ -z "${SIZE}" ]]; then
        exitWithMessage "Error: Zip dSYM file failed."
    else
        echo "✅ Zip dSYM file successfully."
    fi

    echo "Getting upload url..."
    FETCH_URL="https://${COLLECTOR_URL}/analytics/crash/management/symbolFile/uploadUrl/v2"
    FETCH_PARAM='{"file-type":"2","file-name":"'"${DSYM_SYMBOL_ZIP_FILE_NAME}"'","sha256":"'"${SHA}"'","content-length":"'"${SIZE}"'","content-type":"application/octet-stream","user-agent":"curl","clientid":"'"${CLIENT_ID}"'","token":"'"${TOKEN}"'","appid":"'"${APP_ID}"'"}'
    FETCH_RESULT=$(curl -H "Content-type: application/json" -H "appId:${APP_ID}" -X POST -d "$FETCH_PARAM" $FETCH_URL)
    FETCH_CODE=$(get_json_value "${FETCH_RESULT}" "httpCode")
    if [ $FETCH_CODE -eq 200 ]; then
        echo "✅ Get upload url successfully."
    else
        exitWithMessage "Error: get upload url failed.\n${FETCH_RESULT}"
    fi

    UPLOAD_URL=$(get_json_value "${FETCH_RESULT}" "url")
    UPLOAD_OBJ_ID=$(get_json_value "${FETCH_RESULT}" "objectId")
    UPLOAD_ID=$(get_json_value "${FETCH_RESULT}" "id")
    UPLOAD_HEADERS=($(get_json_headers "${FETCH_RESULT}" | tr -d "[],\'"))
    HEADERS=()
    for(( i=0;i<${#UPLOAD_HEADERS[@]};i++)) do
        HEADERS[$i]=$(urldecode "${UPLOAD_HEADERS[i]}")
    done;

    echo "Uploading dSYM zip file..."
    UPLOAD_RESULT=$(curl -sIL -w "%{http_code}" -o /dev/null "${HEADERS[@]/#/-H}" -X PUT -T ${DSYM_SYMBOL_ZIP_FILE} ${UPLOAD_URL})
    if [ $UPLOAD_RESULT -eq 200 ]; then
        echo "✅ Upload dSYM zip file successfully."
    else
        exitWithMessage "Error: upload dSYM zip file failed.\n http code = $UPLOAD_RESULT"
    fi

    echo "sending ack..."
    ACK_URL="https://${COLLECTOR_URL}/analytics/crash/management/symbolFile/uploadAck/v2"
    ACK_PARAM='{"file-type":"2","file-name":"'"${DSYM_SYMBOL_ZIP_FILE_NAME}"'","app_id":"'"${APP_ID}"'","id":"'"${UPLOAD_ID}"'","object-id":"'"${UPLOAD_OBJ_ID}"'","appid":"'"${APP_ID}"'","clientid":"'"${CLIENT_ID}"'","token":"'"${TOKEN}"'"}'
    ACK_RESULT=$(curl -w "%{http_code}" -o /dev/null  -H "Content-type: application/json" -H "appId:${APP_ID}" -X POST -d "$ACK_PARAM" $ACK_URL)
    if [ $ACK_RESULT -eq 200 ]; then
        echo "✅ Send ack successfully."
    else
        exitWithMessage "Error: send ack failed.\n http code = $ACK_RESULT"
    fi

    echo "🎉🎉🎉 All tasks done successfully.\nYou can check it on the AppGallery Connect website."

} 


echo "---------------------------------------------------"
echo "Uploading dSYM to AppGallery Connect Crash Service."
echo "---------------------------------------------------"

INFO_PLIST_FILE="${INFOPLIST_FILE}"
if [ -f "${INFO_PLIST_FILE}" ]; then
    # run the script in Xcode
    ## simulator check
    if [ "$EFFECTIVE_PLATFORM_NAME" == "-iphonesimulator" -a $UPLOAD_SIMULATOR_SYMBOLS -eq 0 ]; then
        exitWithMessage "Warning: Exit due to building for simulator. If you don't want this, set 'UPLOAD_SIMULATOR_SYMBOLS' to 1."
    fi
    ## debug mode check
    if [ "${CONFIGURATION=}" == "Debug" -a $UPLOAD_DEBUG_SYMBOLS -eq 0 ]; then
        exitWithMessage "Warning: Exit due to building for debug mode. If you don't want this, set 'UPLOAD_DEBUG_SYMBOLS' to 1."
    fi
    ## archive mode check
    if [ $UPLOAD_ARCHIVE_ONLY -eq 1 -a "$TARGET_BUILD_DIR" != *"/Archive"* ]; then
        exitWithMessage "Warning: Exit due to building for NOT Archive mode. If you don't want this, set 'UPLOAD_ARCHIVE_ONLY' to 0."
    fi
    upload_dsym ${DWARF_DSYM_FOLDER_PATH}
else
    # run the script in Terminal
    DSYM_FOLDER_PATH="$1"
    if [[ -z "${DSYM_FOLDER_PATH}" ]]; then
        exitWithMessage "Warning: DSYM_FOLDER_PATH not defined. \nusage: dSYMUploader.sh <dsym_folder_path> "
    fi
    upload_dsym ${DSYM_FOLDER_PATH}
fi